<?php
/*
Plugin Name: Discussion Board Pro
Plugin URI: https://wpdiscussionboard.com
Description: Discussion Board - Pro version
Version: 1.9.1
Author: WP Discussion Board
Author URI: https://wpdiscussionboard.com
Text Domain: discussion-board-pro
Domain Path: /languages
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

define( 'CT_DB_PRO_STORE_URL', 'https://wpdiscussionboard.com' );
define( 'CT_DB_PRO_ITEM_NAME', 'Discussion Board Pro' );

function ctdb_pro_load_plugin_textdomain() {
  load_plugin_textdomain( 'discussion-board-pro', FALSE, basename( dirname( __FILE__ ) ) . '/languages/' );
}
add_action( 'plugins_loaded', 'ctdb_pro_load_plugin_textdomain' );

/**
 * Define constants
 **/
if ( ! defined( 'DB_PRO_VERSION' ) ) {
	define( 'DB_PRO_VERSION', '1.9.1' );
}
if ( ! defined( 'DB_PRO_PLUGIN_URL' ) ) {
	define( 'DB_PRO_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
}
if ( ! defined( 'DB_PRO_PLUGIN_DIR' ) ) {
	define( 'DB_PRO_PLUGIN_DIR', dirname( __FILE__ ) );
}

if ( is_admin() ) {
	function ctdb_pro_init() {
		require_once dirname( __FILE__ ) . '/admin/pro-admin-settings.php';
		require_once dirname( __FILE__ ) . '/admin/class-ct-db-pro-admin.php';
		$CT_DB_Pro_Admin = new CT_DB_Pro_Admin();
		$CT_DB_Pro_Admin -> init();
		require_once dirname( __FILE__ ) . '/admin/class-ct-db-pro-admin-notices.php';
		require_once dirname( __FILE__ ) . '/admin/class-ct-db-pro-admin-boards.php';
		require_once dirname( __FILE__ ) . '/admin/class-ct-db-pro-admin-registration.php';
	}
	add_action( 'wpdbd_init', 'ctdb_pro_init' );
}

/**
 * Admin notices
 * @since 1.1.0
 */
function ctdb_pro_admin_notices() {
	// We need to ensure that the free version of the plugin is active
	if ( ! class_exists ( 'CT_DB_Public' ) ) {
		echo '<div class="notice notice-error"><p>' . __( 'Please ensure that you have installed and activated the free version of Discussion Board from the WordPress repository. The Pro version requires it.', 'discussion-board-pro' ) . '</p></div>';
	} else {
		// If it is active, we need to ensure that the minimum version of the free version is 2.1.0
		$plugin_data = get_plugin_data ( WP_PLUGIN_DIR . '/wp-discussion-board/wp-discussion-board.php' );
		if ( $plugin_data && isset ( $plugin_data['Version'] ) ) {
			$version = explode ( '.', $plugin_data['Version'] );
			if ( $version[0] < 3 && $version[1] < 1 ) {
				// We're running less than version 1.3
				echo '<div class="notice notice-error"><p>' . __( 'You are running version ' . $plugin_data['Version'] . ' of the free version of Discussion Board. Please ensure you have the most recent version installed from the WordPress plugin repository to avoid any possible conflicts.', 'discussion-board-pro' ) . '</p></div>';
			}
		} else {
			// Can't retrieve the plugin for some reason
			echo '<div class="notice notice-warning"><p>' . __( 'We can\'t detect the version of Discussion Board that you are running. Please ensure you have the most recent version installed from the WordPress plugin repository. The Pro version requires it.', 'discussion-board-pro' ) . '</p></div>';
		}
	}
}
add_action ( 'admin_notices', 'ctdb_pro_admin_notices' );

/**
 * Include the pro functions.
 */
function ctdb_pro_public_init() {

	require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-public.php';
	require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-categories.php';
	require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-boards.php';
	require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-new-topic.php';
	require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-registration.php';
	require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-profiles.php';
	require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-skins.php';
	require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-status.php';
	require_once dirname( __FILE__ ) . '/inc/customizer.php';
	require_once dirname( __FILE__ ) . '/inc/functions-profiles.php';
	require_once dirname( __FILE__ ) . '/inc/functions-boards.php';
	require_once dirname( __FILE__ ) . '/inc/functions-breadcrumbs.php';
	require_once dirname( __FILE__ ) . '/inc/functions-messages.php';
	require_once dirname( __FILE__ ) . '/inc/functions-pro.php';
	require_once dirname( __FILE__ ) . '/inc/functions-registration.php';
	require_once dirname( __FILE__ ) . '/inc/functions-status.php';
	require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-uploader.php';
	require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-template-loader.php';
	$CT_DB_Pro_Public = new CT_DB_Pro_Public();
	$CT_DB_Pro_Public -> init();
	$CT_DB_Pro_Template_Loader = new CT_DB_Pro_Template_Loader();
	$CT_DB_Pro_Template_Loader -> init();
	$CT_DB_Pro_Uploader = new CT_DB_Pro_Uploader();
	$CT_DB_Pro_Uploader -> init();

	// Enable following
	$options = get_option( 'ctdb_followers_settings' );
	if( isset( $options['enable_topic_following'] ) && ! empty( $options['enable_topic_following'] ) ) {
		require_once dirname( __FILE__ ) . '/public/class-ct-db-pro-follow.php';
	}

	// Add the widgets
	require_once dirname( __FILE__ ) . '/widgets/class-ct-db-widget-login.php';
	require_once dirname( __FILE__ ) . '/widgets/class-ct-db-widget-recent-topics.php';
	require_once dirname( __FILE__ ) . '/widgets/class-ct-db-widget-recent-replies.php';
	require_once dirname( __FILE__ ) . '/widgets/class-ct-db-widget-top-posters.php';
	require_once dirname( __FILE__ ) . '/widgets/class-ct-db-widget-top-commenters.php';
}
// Only include if free version is active
add_action ( 'ct_db_public_init', 'ctdb_pro_public_init', 10 );


if( ! class_exists( 'EDD_SL_Plugin_Updater' ) ) {
	// load our custom updater
	include( dirname( __FILE__ ) . '/inc/updater/EDD_SL_Plugin_Updater.php' );
}

function ct_db_pro_updater() {
	// retrieve our license key from the DB
	$license_key = trim( get_option( 'ct_db_pro_license_key' ) );

	// setup the updater
	$edd_updater = new EDD_SL_Plugin_Updater( CT_DB_PRO_STORE_URL, __FILE__, array(
			'version' 	=> DB_PRO_VERSION, 		// current version number
			'license' 	=> $license_key, 		// license key (used get_option above to retrieve from DB)
			'item_name' => CT_DB_PRO_ITEM_NAME, // name of this plugin
			'author' 	=> 'WP Discussion Board'  	// author of this plugin
		)
	);

}
add_action( 'admin_init', 'ct_db_pro_updater', 0 );


/************************************
* Options page
*************************************/
// Add the submenu item
function ctdb_add_license_submenu() {
	add_submenu_page( 'edit.php?post_type=discussion-topics', __( 'License', 'discussion-board-pro' ), __( 'License', 'discussion-board-pro' ), 'manage_options', 'discussion_board_license', 'ct_db_pro_license_page' );
}
add_action ( 'admin_menu', 'ctdb_add_license_submenu' );

function ct_db_pro_license_page() {
	$license 	= get_option( 'ct_db_pro_license_key' );
	$status 	= get_option( 'ct_db_pro_license_status' );
	?>
	<div class="wrap">
		<h1><?php _e( 'License', 'discussion-board-pro' ); ?></h1>
		<?php settings_errors(); ?>
		<form method="post" action="options.php">

			<?php settings_fields( 'ct_db_pro_license' ); ?>

			<table class="form-table">
				<tbody>
					<tr valign="top">
						<th scope="row" valign="top">
							<?php _e( 'License Key', 'discussion-board-pro' ); ?>
						</th>
						<td>
							<input id="ct_db_pro_license_key" name="ct_db_pro_license_key" type="text" class="regular-text" value="<?php esc_attr_e( $license ); ?>" />
							<p class="description" for="ct_db_pro_license_key"><?php _e( 'Enter your license key', 'discussion-board-pro' ); ?></p>
						</td>
					</tr>
					<tr valign="top">
						<th scope="row" valign="top">
							<?php _e( 'License Status', 'discussion-board-pro' ); ?>
						</th>
						<td>
							<?php if( $status !== false && $status == 'valid' ) { ?>
								<span style="color: green;"><?php _e( 'Active', 'discussion-board-pro' ); ?></span>
							<?php } else { ?>
								<span style="color: red;"><?php _e( 'Inactive', 'discussion-board-pro' ); ?></span>
							<?php } ?>
						</td>
					</tr>
					<?php if( false !== $license ) { ?>

					<?php } ?>
				</tbody>
			</table>
			<?php if( $status !== false && $status == 'valid' ) { ?>
				<?php wp_nonce_field( 'ct_db_pro_license_nonce', 'ct_db_pro_license_nonce' ); ?>
				<input type="submit" class="button-secondary" name="dbpro_license_deactivate" value="<?php _e( 'Deactivate License', 'discussion-board-pro' ); ?>"/>
			<?php } else if( ! empty( $license ) ) {
				wp_nonce_field( 'ct_db_pro_license_nonce', 'ct_db_pro_license_nonce' ); ?>
				<input type="submit" class="button-secondary" name="dbpro_license_activate" value="<?php _e( 'Activate License', 'discussion-board-pro' ); ?>"/>
			<?php } ?>
			<?php submit_button(); ?>

		</form>
	<?php
}

function ct_db_pro_register_option() {
	// creates our settings in the options table
	register_setting('ct_db_pro_license', 'ct_db_pro_license_key', 'ct_db_pro_sanitize_license' );
}
add_action('admin_init', 'ct_db_pro_register_option');

function ct_db_pro_sanitize_license( $new ) {
	$old = get_option( 'ct_db_pro_license_key' );
	if( $old && $old != $new ) {
		delete_option( 'ct_db_pro_license_status' ); // new license has been entered, so must reactivate
	}
	return $new;
}

/************************************
* this illustrates how to activate
* a license key
*************************************/
function ct_db_pro_activate_license() {

	// listen for our activate button to be clicked
	if( isset( $_POST['dbpro_license_activate'] ) ) {

		// run a quick security check
	 	if( ! check_admin_referer( 'ct_db_pro_license_nonce', 'ct_db_pro_license_nonce' ) )
			return; // get out if we didn't click the Activate button

		// retrieve the license from the database
		$license = trim( get_option( 'ct_db_pro_license_key' ) );

		// data to send in our API request
		$api_params = array(
			'edd_action'=> 'activate_license',
			'license' 	=> $license,
			'item_name' => urlencode( CT_DB_PRO_ITEM_NAME ), // the name of our product in EDD
			'url'       => home_url()
		);

		// Call the custom API.
		$response = wp_remote_post( CT_DB_PRO_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay
		if ( is_wp_error( $response ) )
			return false;

		// decode the license data
		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		if( $license_data->license == 'invalid' ) {
		//	add_action( 'admin_notices', 'ctdb_invalid_license' );
			update_option( 'ct_db_pro_license_status', $license_data->license );
			update_option( 'ct_db_pro_license_level', 'invalid' );
		} else {
			update_option( 'ct_db_pro_license_status', $license_data->license );
			update_option( 'ct_db_pro_license_level', $license_data->license_limit );
		}
	}
}
add_action( 'admin_init', 'ct_db_pro_activate_license' );


/***********************************************
* Illustrates how to deactivate a license key.
* This will decrease the site count
***********************************************/

function ct_db_pro_deactivate_license() {

	// listen for our activate button to be clicked
	if( isset( $_POST['dbpro_license_deactivate'] ) ) {

		// run a quick security check
	 	if( ! check_admin_referer( 'ct_db_pro_license_nonce', 'ct_db_pro_license_nonce' ) )
			return; // get out if we didn't click the Activate button

		// retrieve the license from the database
		$license = trim( get_option( 'ct_db_pro_license_key' ) );

		// data to send in our API request
		$api_params = array(
			'edd_action'=> 'deactivate_license',
			'license' 	=> $license,
			'item_name' => urlencode( CT_DB_PRO_ITEM_NAME ), // the name of our product in EDD
			'url'       => home_url()
		);

		// Call the custom API.
		$response = wp_remote_post( CT_DB_PRO_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay
		if ( is_wp_error( $response ) )
			return false;

		// decode the license data
		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// $license_data->license will be either "deactivated" or "failed"
		if( $license_data->license == 'deactivated' ) {
			delete_option( 'ct_db_pro_license_status' );
			delete_option( 'ct_db_pro_license_level' );
		}

	}

}
add_action( 'admin_init', 'ct_db_pro_deactivate_license' );

/**
 * Returns how many activations are available to this license
 */
function ctdb_pro_check_license_option() {

	global $wp_version;

	$license = trim( get_option( 'ct_db_pro_license_key' ) );

	$api_params = array(
		'edd_action' => 'check_license',
		'license' => $license,
		'item_name' => urlencode( CT_DB_PRO_ITEM_NAME ),
		'url'       => home_url()
	);

	// Call the custom API.
	$response = wp_remote_post( CT_DB_PRO_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

	if ( is_wp_error( $response ) )
		return false;

	$license_data = json_decode( wp_remote_retrieve_body( $response ) );

	if( isset( $license_data->license_limit ) ) {
		return $license_data->license_limit;
		exit;
	} else {
		return 'invalid';
		exit;
	}
}

/**
 * Include the TGM_Plugin_Activation class.
 */
require_once dirname( __FILE__ ) . '/inc/tgm/class-tgm-plugin-activation.php';

add_action( 'tgmpa_register', 'ctdb_pro_register_required_plugins' );

/**
 * Register the required plugins for this theme.
 *
 * In this example, we register five plugins:
 * - one included with the TGMPA library
 * - two from an external source, one from an arbitrary source, one from a GitHub repository
 * - two from the .org repo, where one demonstrates the use of the `is_callable` argument
 *
 * The variable passed to tgmpa_register_plugins() should be an array of plugin
 * arrays.
 *
 * This function is hooked into tgmpa_init, which is fired within the
 * TGM_Plugin_Activation class constructor.
 */
function ctdb_pro_register_required_plugins() {
	/*
	 * Array of plugin arrays. Required keys are name and slug.
	 * If the source is NOT from the .org repo, then source is also required.
	 */
	$plugins = array(

		// Plugins from the WordPress Plugin Repository
		array(
			'name'      => 'Discussion Board',
			'slug'      => 'wp-discussion-board',
			'required'  => true,
			'version'		=> '2.3.4'
		)

	);

	/**
	 * Array of configuration settings. Amend each line as needed.
	 *
	 * TGMPA will start providing localized text strings soon. If you already have translations of our standard
	 * strings available, please help us make TGMPA even better by giving us access to these translations or by
	 * sending in a pull-request with .po file(s) with the translations.
	 *
	 * Only uncomment the strings in the config array if you want to customize the strings.
	 */
	$config = array(
		'id'           => 'tgmpa',                 // Unique ID for hashing notices for multiple instances of TGMPA.
		'default_path' => '',                      // Default absolute path to bundled plugins.
		'menu'         => 'tgmpa-install-plugins', // Menu slug.
		'parent_slug'  => 'plugins.php',            // Parent menu slug.
		'capability'   => 'edit_theme_options',    // Capability needed to view plugin install page, should be a capability associated with the parent menu used.
		'has_notices'  => true,                    // Show admin notices or not.
		'dismissable'  => true,                    // If false, a user cannot dismiss the nag message.
		'dismiss_msg'  => '',                      // If 'dismissable' is false, this message will be output at top of nag.
		'is_automatic' => false,                   // Automatically activate plugins after installation or not.
		'message'      => '',                      // Message to output right before the plugins table.
	);

	tgmpa( $plugins, $config );
}
