<?php
/**
 * Discussion Board Boards class
 * @since 1.5.0
 * Allows for multiple boards on different subjects
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Plugin public class
 **/
class CT_DB_Pro_Boards extends CT_DB_Public {

	public function __construct() {
		//
	}

	/**
	 * Initialize the class and start calling our hooks and filters
	 * @since 1.5.0
	 */
	public function init() {

		add_action( 'init', array( $this, 'register_taxonomy' ) );
		add_action( 'wp_head', array( $this, 'set_board_id' ), 99, 1 );

		add_filter( 'query_vars', array( $this, 'add_query_var' ) );

		add_action( 'admin_menu', array( $this, 'remove_meta_box' ) );
		add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ) );
		add_action( 'save_post', array( $this, 'save_board' ) );

		add_filter( 'ctdb_topics_form_before_submit', array( $this, 'topic_form_filter' ), 10 );
		add_filter( 'ctdb_post_args', array( $this, 'topic_form_submission_filter' ), 20 );
		add_filter( 'ctdb_new_topic_form_validation', array( $this, 'validate_board_submission' ) );

		// Filter the new_topic_button shortcode
		add_filter( 'ctdb_new_topic_button_url', array( $this, 'filter_new_topic_button_url' ), 10, 3 );
		add_shortcode( 'discussion_board', array( $this, 'discussion_board_shortcode' ) );
		add_shortcode( 'discussion_board_children', array( $this, 'discussion_board_children_shortcode' ) );

		// Add a new topic button to the taxonomy-board.php template
		/**
		 * @todo remove this, new topic button is now added in board summary footer on taxonomy-board.php
		 */
		$options = get_option ( 'ctdb_boards_settings' );
		if( isset( $options['new_topic_button_archive'] ) ) {
			if( $options['new_topic_button_archive'] == 'top' ) {
			//	add_action( 'ctdb_board_content', array( $this, 'add_button_taxonomy_board' ), 5 );
			} else if( $options['new_topic_button_archive'] == 'bottom' ) {
			//	add_action( 'ctdb_board_content', array( $this, 'add_button_taxonomy_board' ), 999 );
			}
		}

		// Update the child board select field
		add_action( 'wp_ajax_update_child_board_select', array( $this, 'update_child_board_select' ) );
		add_action( 'wp_ajax_nopriv_update_child_board_select', array( $this, 'update_child_board_select' ) );

		// Update the follow/unfollow status for boards
		add_action( 'wp_ajax_update_board_follow_status', array( $this, 'update_board_follow_status' ) );

	}

	/**
	 * Add our $board_id var to the public vars
	 * @since 1.5.0
	 */
	public function add_query_var( $vars ) {
		$vars[] = 'board_id';
		return $vars;
	}

	/**
	 * Set a field with the Board ID
	 * Just using this as a double-check for the get_query_var
	 * @since 1.5.0
	 */
	public function set_board_id( $form ) {
		echo '<input type="hidden" name="ctdb_board_id" id="ctdb_board_id" value="' . esc_attr( get_query_var( 'board_id' ) ) . '">';
	}

	// Add boards to New Topic Form
	public function topic_form_filter( $form ) {
		$board_id = get_query_var( 'board_id' );

		// Check if a board ID has been passed in
		if( ! empty( $board_id ) ) {
			$board_id = intval( get_query_var( 'board_id' ) );
		} else {
			// If there is no board ID, add a select so that the user can pick one
			$options = get_option( 'ctdb_boards_settings' );

			// Add a select box for categories
			// @todo Display as checkboxes / radio buttons / multiselect ?
			if( ! empty( $options['enable_boards'] ) ) {
				$terms = get_terms ( 'board', array ( 'hide_empty' => 0, 'parent' => 0 ) );
				$term_select = '<div><select name="new_topic_board" id="new_topic_board" class="postform required ctdb_new_topic_select" tabindex="902">';
				$term_select .= '<option value="-1">' . __ ( 'Select a board', 'discussion-board-pro' ) . '</option>';
				// Add terms if they exist
				if ( ! empty ( $terms ) && ! is_wp_error ( $terms ) ) {
					foreach ( $terms as $term ) {
						// Check if we are allowed to post to this board
						if( ctdb_is_posting_permitted_by_board( $term->term_id ) ) {
							$term_select .= '<option value="' . $term->term_id . '">' . $term->name . '</option>';
						}

					}
				}
				$term_select .= '</select></div>';

				// Add child board select box too
				$term_select .= '<div><select style="display: none;" name="new_topic_child_board" id="new_topic_child_board" class="postform required ctdb_new_topic_select">';
				$term_select .= '</select></div>';

				$term_select .= "<script>
					jQuery(document).ready(function($){
						$('#new_topic_board').on('change',function(){
							var board_id = $(this).val();
							var security = $('#new_topic').val();
							$.ajax({
								type: 'POST',
								url: ajax_object.ajaxurl,
								data: {
									board_id: board_id,
									security: security,
									action: 'update_child_board_select',
								},
								success: function(response){
									if(response) {
										$('#new_topic_child_board').fadeIn();
										$('#new_topic_child_board').html( response );
									}
								}
							});
						});
					});
					</script>";
			}
		}



		// Add the select box to the $fields
		if( isset( $term_select ) && $term_select != '' ) {
			$form['fields']['term_select'] = $term_select;
		}

		return $form;

	}

	/**
	 * Update the child board select depending on value of parent board
	 * @since 1.7.0
	*/
	public function update_child_board_select() {
		if( ! isset( $_POST['security'] ) || ! wp_verify_nonce( $_POST['security'], 'display_new_topic_form' ) ) {
			die();
		}
		$board_id = $_POST['board_id'];
		$terms = get_terms( 'board', array ( 'hide_empty' => 0, 'parent' => $board_id ) );
		$term_select = '';
		if ( ! empty ( $terms ) && ! is_wp_error ( $terms ) ) {
			foreach ( $terms as $term ) {
				// Check if we are allowed to post to this board
				if( ctdb_is_posting_permitted_by_board( $term->term_id ) ) {
					$term_select .= '<option value="' . $term->term_id . '">' . $term->name . '</option>';
				}
			}
		}
		wp_send_json( $term_select );
	}

	/**
	 * Filter the submission form
	 * Add taxonomy term for board
	 * @hooked to ctdb_post_args	Priority 20	After topic_categories are filtered
	 * @since 1.5.0
	*/
	public function topic_form_submission_filter( $post_args ) {
		// Check if a board ID has been passed in
		$board_id = get_query_var( 'board_id' );
		if( ! empty( $board_id ) ) {
			$board_id = intval( get_query_var( 'board_id' ) );
		} else if( isset( $_POST['new_topic_board'] ) ) {
			// Else, use the ID from the new topic select
			// Check for taxonomy
			$board_id = intval( $_POST['new_topic_board'] );
			// If there's a child board, use that ID
			if( isset( $_POST['new_topic_child_board'] ) ) {
				$board_id = intval( $_POST['new_topic_child_board'] );
			}
		}

		// If a board ID has been set, pass it to the wp_insert_post $args
		if( ! empty( $board_id ) ) {
			if( isset( $post_args['tax_input'] ) && is_array( $post_args['tax_input'] ) ) {
				// If there's already something in tax_input (e.g. category), we add a new element to the array
				$post_args['tax_input']['board'] = $board_id;
			} else {
				// Otherwise we just make a new array
				$post_args['tax_input'] = array (
					'board'	=> $board_id
				);
			}
		}

		return $post_args;
	}

	/**
	 * Filter the validation params to ensure board is set
	 * @hooked to ctdb_new_topic_form_validation
	 * @since 1.5.0
	*/
	public function validate_board_submission( $error_params ) {
		// Check if a board ID has been passed in
		// If not, we need to ensure a board has been specified
		if( ! isset( $_GET['board_id'] ) ) {
			$error_params['new_topic_board'] = array(
				'id'			=> 'new_topic_board', // The form element we're evaluating
				'check'			=> -1, // The value that will throw the error
				'response'		=> __( 'Please ensure you allocate this topic to a Board.', 'discussion-board-pro' ), // The error message if the element is empty
				'topic_element'	=> 'board' // The element of the $post_args to be populated with this element's value
			);
		}

		return $error_params;
	}

	/**
	 * The shortcode to display the board
	 * Uses the discussion_topics shortcode from the free version
	 * @since 1.5.0
	 *
	 * @since 1.7.0
	 * Has option to display child boards
	 */
	public function discussion_board_shortcode( $atts ) {

		global $CT_DB_Public, $CT_DB_Front_End;

		if ( empty( $atts ) ) {
			$atts = array();
		}

		// If we're on a board archive page, show the board.
		if ( is_tax( 'board' ) ) {
			$board         = get_term_by( 'slug', get_query_var( 'board' ), 'board' );
			$atts['board'] = $board->term_id;
		}

		if ( ! empty( $atts['board'] ) ) {
			$board_id = intval( $atts['board'] );
		}

		if ( empty( $board_id ) ) {
			// No board ID set
			return;
		}

		$options = get_option( 'ctdb_boards_settings' );
		if( ! isset( $options['enable_boards'] ) || empty( $options['enable_boards'] ) ) {
			// Boards are not enabled
			// Show admin a message to let them know
			if( current_user_can( 'update_plugins' ) ) {
				return '<p class="ctdb-admin-message">' . __( 'Site Admin: the Boards feature has been disabled in Discussion Board Pro so this shortcode will not display. Non-admins will not see this message.', 'discussion-board-pro' ) . '</p>';
			} else {
				return;
			}

		}

		// User not allowed
		if( ctdb_is_user_permitted() != true ) {
			return $CT_DB_Front_End->return_access_restricted_message();
		}

		$number_topics = $options['topics_in_boards'];

		// Check how many topics to display
		if( ! isset( $atts['number'] ) ) {
			$atts['number'] = $number_topics;
		}

		// Don't display the navigation
		$atts['show_nav'] = false;

		// Get the term object
		$term = get_term( $board_id, 'board' );
		if( empty( $term ) ) {
			return;
		}

		$content = '';

		if( isset( $term->name ) ) {
			$content .= '<div class="ctdb-board-outer">';
				$content .= '<div class="ctdb-board-wrapper">';

				// Get the header for the board
					$content .= ctdb_get_board_header( $board_id, $term );

					$content .= ctdb_get_board_meta( $board_id, $term );

				$content .= '</div><!-- ctdb-board-wrapper -->';

			// Filter the board header and meta
			// $content = apply_filters( 'ctdb_filter_board_header', $content, $term, $board_id, $fields, $all_fields );

			// This gets the discussion_topics shortcode content
			// Pass a class so that we don't have additional margin-bottom
			$content .= $CT_DB_Front_End->display_all_topics( $atts, 'ctdb-board-no-topics' );

			$content .= ctdb_get_board_footer( $board_id, $term );

			$content .= '</div><!-- ctdb-board-outer -->';

			wp_reset_query();

		}

		return $content;
	}

	/**
	 * The shortcode to display board children
	 * @since 1.7.0
	 */
	public function discussion_board_children_shortcode( $atts ) {
		global $CT_DB_Public, $CT_DB_Front_End;

		if( isset( $atts['board'] ) ) {
			$board_id = intval( $atts['board'] );
		} else {
			// No board ID set
			return;
		}

		$options = get_option( 'ctdb_boards_settings' );
		if( ! isset( $options['enable_boards'] ) || empty( $options['enable_boards'] ) ) {
			// Boards are not enabled
			// Show admin a message to let them know
			if( current_user_can( 'update_plugins' ) ) {
				return '<p class="ctdb-admin-message">' . __( 'Site Admin: the Boards feature has been disabled in Discussion Board Pro so this shortcode will not display. Non-admins will not see this message.', 'discussion-board-pro' ) . '</p>';
			} else {
				return;
			}
		}

		// User not allowed
		if( ctdb_is_user_permitted() != true ) {
			return $CT_DB_Front_End->return_access_restricted_message();
		}

		$number_topics = $options['topics_in_boards'];

		// Check how many topics to display
		if( ! isset( $atts['number'] ) ) {
			$atts['number'] = $number_topics;
		}

		// Don't display the navigation
		$atts['show_nav'] = false;

		// Get the term object
		$term = get_term( $board_id, 'board' );
		if( empty( $term ) ) {
			return;
		}

		$content = '';

		if( $term->name ) {
			$content .= '<div class="ctdb-board-outer">';
				$content .= '<div class="ctdb-board-wrapper">';
				if( $term->name ) {
					$content .= '<div class="ctdb-board-header">';
						$content .= '<div class="ctdb-board-header-inner">';
						if( ctdb_is_user_permitted() == true && $term->count > 0 ) {
							// Site title and link
							$content .= sprintf( '<h2 class="%s"><a href="%s">%s</a></h2>',
								esc_attr( 'board-title' ),
								esc_url( get_term_link( $board_id ) ),
								esc_textarea( $term->name )
							);
						} else {
							// Site title only if no topics exist yet or user is not permitted
							$content .= sprintf( '<h2 class="%s">%s</h2>',
								esc_attr( 'board-title' ),
								esc_textarea( $term->name )
							);
						}

						$content .= '</div><!-- ctdb-board-header-inner -->';
					$content .= '</div><!-- ctdb-board-header -->';
				}

				$content .= '<div class="ctdb-board-meta-wrap">';
					$content .= '<div class="ctdb-board-thumb">';
						$image_id = get_term_meta( $term->term_id, 'ctdb-board-image-id', true );
						$content .= wp_get_attachment_image( $image_id );
					//	$content .= wpautop( $term->description );
					$content .= '</div><!-- ctdb-board-thumb -->';
					$content .= '<div class="ctdb-board-description">';
						$content .= wpautop( $term->description );
					$content .= '</div><!-- ctdb-board-description -->';
					$content .= '<div class="ctdb-board-meta">';
						// Get selected fields to display
						$options = get_option( 'ctdb_boards_settings' );
						$fields = array();
						if( isset( $options['boards_fields'] ) ) {
							$fields = $options['boards_fields'];
						}

						// Get list of all available fields
						$all_fields = ctdb_boards_fields();

						// Iterate through selected fields
						if( ! empty( $fields ) && ! empty( $all_fields ) ) {
							foreach( $fields as $field ) {

								// If the field is available, add the content
								if( isset( $all_fields[$field]['callback'] ) && function_exists( $all_fields[$field]['callback'] ) ) {
									$content .= '<div class="ctdb-board-meta-inner ctdb-board-meta-' . esc_attr( $field ) . '">';
										$label = $all_fields[$field]['label'];
										$content .= call_user_func( $all_fields[$field]['callback'], $term, $term->term_id, $label );
									$content .= '</div>';
								}

							}
						}
					$content .= '</div><!-- ctdb-board-meta -->';
				$content .= '</div><!-- ctdb-board-meta-wrap -->';

			$content .= '</div><!-- ctdb-board-wrapper -->';

			// Filter the board header and meta
			$content = apply_filters( 'ctdb_filter_board_header', $content, $term, $board_id, $fields, $all_fields );

			// Get all child boards
			$content .= $this->get_child_boards( $atts );

			// $content .= $this->get_board_footer( $board_id, $term );

			$content .= '</div><!-- ctdb-board-outer -->';

			wp_reset_query();

		}

		return $content;
	}



	/**
	 * Get the child boards for specified board
	 * @since 1.7.0
	 * @param $atts Shortcode atts
	 */
	public function get_child_boards( $atts ) {
		$return = sprintf( '<p>%s</p>', __( 'There are no child boards to display', 'discussion-board-pro' ) );
		// Get the board ID
		$board_id = intval( $atts['board'] );
		if( isset( $atts['ids'] ) ) {
			$children = explode( ',', trim( $atts['ids'] ) );
		} else {
			$children = get_term_children( $board_id, 'board' );
		}

		if( ! empty( $children ) ) {
			$return = $this->get_child_board_cols( $children, $atts );
			return $return;
		}
		return $return;
	}

	public function get_child_board_cols( $children, $atts ) {
		global $post;

		// These are the columns that will appear in our table
		if( empty( $atts['cols'] ) ) {
			$cols = array( 'avatar', 'board', 'replies', 'started' );
		} else {
			$cols = explode( ',', $atts['cols'] );
		}

		$colcount = 0;

		// Column titles: keys should match $cols elements above
		$titles = array(
			'avatar'		=> '&nbsp;',
			'board'			=> __( 'Board', 'discussion-board-pro' ),
			'topics'		=> __( 'Topics', 'discussion-board-pro' ),
			'replies' 		=> __( 'Replies', 'discussion-board-pro' ),
			'started'		=> __( 'Since', 'discussion-board-pro' ),
		//	'posted-by'		=> __( 'Posted by: ', 'wp-discussion-board' ),
			'voices'		=> __( 'Voices', 'discussion-board-pro' )
		);

		// You can filter the titles here
		$titles = apply_filters( 'ctdb_topic_titles', $titles );

		// @since 1.6.1
		// Build the column titles with this array
		$title_fields = array();
		if( ! empty( $cols ) ) {
			$colcount = count( $cols );
			foreach( $cols as $col ) {
				// Check the col is valid
				if( isset( $titles[$col] ) ) {
					$title_fields[$col] = '<li class="ctdb-topic-table-item ctdb-topic-table-' . esc_attr( $col ) . '">' . $titles[$col] . '</li>';
				}
			}
		}

		// We can filter these fields if needed
		$title_fields = apply_filters( 'ctdb_topic_title_fields', $title_fields, $cols );

		$output = '';

		// The HTML for the column titles
		$output .= '<ul class="ctdb-topic-table ctdb-topic-table-col-' . intval( $colcount ) . '">';
			$output .= '<li class="ctdb-topic-table-header">';
				$output .= '<ul class="ctdb-topic-table-row ctdb-topic-table-header-row">';

					if( ! empty( $title_fields ) ) {
						$output .= join( '', $title_fields );
					}

				$output .= '</ul><!-- ctdb-topic-table-row -->';
			$output .= '</li><!-- ctdb-topic-table-header -->';

			// Now build the table rows, i.e. the topics
			$output .= '<li class="ctdb-topic-table-body">';

			$body_fields = array();

			foreach( $children as $child_id ) {

				$body_fields[$child_id] = '<ul class="ctdb-topic-table-row">';

				if( ! empty( $cols ) ) {

					foreach( $cols as $col ) {

						if( isset( $titles[$col] ) ) {

							$term = get_term_by( 'id', $child_id, 'board' );

							$body_fields[$child_id] .= '<li class="ctdb-topic-table-item ctdb-topic-table-' . esc_attr( $col ) . '">';

							if( $col == 'avatar' ) {
								$image_id = get_term_meta( $child_id, 'ctdb-board-image-id', true );
								$body_fields[$child_id] .= wp_get_attachment_image( $image_id );
							} else if( $col == 'board' ) {

								$link = get_term_link( ( int )$child_id );
								if( ! is_wp_error( $link ) ) {
									$body_fields[$child_id] .= sprintf(
										'<p><a href="%1$s">%2$s</a></p>',
										esc_url( get_term_link( ( int )$child_id ) ),
										esc_html( $term-> name )
									);
								}

								// We output hidden metafields here, displayed on mobile
								$body_fields[$child_id] .= '<div class="ctdb-topic-mobile-metafields">';
									if( in_array( 'replies', $cols ) ) {
										$body_fields[$child_id] .= '<div>';
										$body_fields[$child_id] .= __( 'Replies', 'wp-discussion-board' );
										$body_fields[$child_id] .= ': '. ctdb_get_board_replies_count( $child_id );
										$body_fields[$child_id] .= '</div>';
									}
									if( in_array( 'started', $cols ) ) {
										$body_fields[$child_id] .= '<div>';
										$body_fields[$child_id] .= __( 'Started', 'wp-discussion-board' );
										$body_fields[$child_id] .= ': '. ctdb_get_board_freshness( $child_id );
										$body_fields[$child_id] .= '</div>';
									}
									if( in_array( 'voices', $cols ) ) {
										$body_fields[$child_id] .= '<div>';
										$body_fields[$child_id] .= __( 'Voices', 'wp-discussion-board' );
										$body_fields[$child_id] .= ': '. absint( ctdb_get_board_voices_count( $child_id ) );
										$body_fields[$child_id] .= '</div>';
									}
									$mobile_body_fields = '';
									$mobile_body_fields = apply_filters( 'ctdb_mobile_body_fields', $mobile_body_fields, $post->ID, $cols );
									$body_fields[$child_id] .= $mobile_body_fields;
								$body_fields[$child_id] .= '</div>';

							} else if( $col == 'topics' ) {
								$body_fields[$child_id] .= ctdb_get_board_topics_count( $term );
							} else if( $col == 'replies' ) {
								$body_fields[$child_id] .= ctdb_get_board_replies_count( $child_id );
							} else if( $col == 'freshness' ) {
								$body_fields[$child_id] .= ctdb_get_board_freshness( $child_id );
							} else if( $col == 'voices' ) {
								$body_fields[$child_id] .= absint( ctdb_get_board_voices_count( $child_id ) );
							}

							// Filter here for additional columns
							$body_fields = apply_filters( 'ctdb_topic_column', $body_fields, $child_id, $col );

							$body_fields[$child_id] .= '</li>';

						}

					}

				}

				$body_fields[$child_id] .= '</ul><!-- ctdb-topic-table-row -->';

			} // foreach

			if( ! empty( $body_fields ) ) {
				$output .= join( '', $body_fields );
			}


			$output .= '</li><!-- .ctdb-topic -->';
		$output .= '</ul><!-- ctdb-topic-table-body -->';

		return $output;
	}

	public function get_child_board_headings() {
		global $post;

		// These are the columns that will appear in our table
		if( empty( $cols ) ) {
			$cols = array( 'avatar', 'topic', 'replies', 'started' );
		}

		$colcount = 0;

		// Column titles: keys should match $cols elements above
		$titles = array(
			'avatar'		=> '&nbsp;',
			'topic'			=> __( 'Topic', 'wp-discussion-board' ),
			'replies' 		=> __( 'Replies', 'wp-discussion-board' ),
			'started'		=> __( 'Since', 'wp-discussion-board' ),
			'posted-by'		=> __( 'Posted by: ', 'wp-discussion-board' ),
			'voices'		=> __( 'Voices', 'wp-discussion-board' )
		);

		// You can filter the titles here
		$titles = apply_filters( 'ctdb_topic_titles', $titles );

		// @since 1.6.1
		// Build the column titles with this array
		$title_fields = array();
		if( ! empty( $cols ) ) {
			$colcount = count( $cols );
			foreach( $cols as $col ) {
				// Check the col is valid
				if( isset( $titles[$col] ) ) {
					$title_fields[$col] = '<li class="ctdb-topic-table-item ctdb-topic-table-' . esc_attr( $col ) . '">' . $titles[$col] . '</li>';
				}
			}
		}

		// We can filter these fields if needed
		$title_fields = apply_filters( 'ctdb_topic_title_fields', $title_fields, $cols );

		// Filter the entire output before running code below
		// Allows us to build our own markup
		$output = '';
		$output = apply_filters( 'ctdb_filter_table_layout', $output, $topics, $titles, $cols );

		// If the output hasn't been pre-filtered, then build the markup
		if( empty( $output ) ) {

			// The HTML for the column titles
			$output .= '<ul class="ctdb-topic-table ctdb-topic-table-col-' . intval( $colcount ) . '">';
				$output .= '<li class="ctdb-topic-table-header">';
					$output .= '<ul class="ctdb-topic-table-row ctdb-topic-table-header-row">';

						if( ! empty( $title_fields ) ) {
							$output .= join( '', $title_fields );
						}

					$output .= '</ul><!-- ctdb-topic-table-row -->';
				$output .= '</li><!-- ctdb-topic-table-header -->';

				// Now build the table rows, i.e. the topics
				$output .= '<li class="ctdb-topic-table-body">';

				$body_fields = array();

				// Use this filter to insert your own body fields before the query runs
				// Avoids running the query twice
				$body_fields = apply_filters( 'ctdb_topic_body_fields_before_query', $body_fields, $topics, $titles, $cols );

				if( empty( $body_fields ) ) {

					while( $topics->have_posts() ) : $topics->the_post();

						$body_fields[$post->ID] = '<ul class="ctdb-topic-table-row">';

						if( ! empty( $cols ) ) {

							foreach( $cols as $col ) {

								if( isset( $titles[$col] ) ) {

									$body_fields[$post->ID] .= '<li class="ctdb-topic-table-item ctdb-topic-table-' . esc_attr( $col ) . '">';
									// Get some metafields for the topic
									// Number of comments
									$comments_number = get_comments_number();
									// Start time
									$started_time = '<time class="comment-timeago" datetime="' . date( 'c', strtotime( get_the_date() . ' ' . get_the_time() ) ) . '">' . get_the_date() . '</time>';
									// Voices
									$comments = get_comments( $post );
									$CT_DB_Skins = new CT_DB_Skins();
									$voices = $CT_DB_Skins->count_voices( $comments );

									if( $col == 'avatar' ) {
										$body_fields[$post->ID] .= get_avatar( get_the_author_meta( 'ID' ), 48 );
									} else if( $col == 'topic' ) {
										$url = get_permalink();
										$body_fields[$post->ID] .= sprintf(
											'<p><a href="%1$s">%2$s</a></p>',
											esc_url( $url ),
											get_the_title()
										);
										$author = get_the_author();
										$author = apply_filters( 'ctdb_author_name', $author );
										$body_fields[$post->ID] .= '<span class="ctdb-topic-table-posted-by">' . $titles['posted-by'] . $author . '</span>';
										// We output hidden metafields here, displayed on mobile

										$body_fields[$post->ID] .= '<div class="ctdb-topic-mobile-metafields">';
											if( in_array( 'replies', $cols ) ) {
												$body_fields[$post->ID] .= '<div>';
												$body_fields[$post->ID] .= __( 'Replies', 'wp-discussion-board' );
												$body_fields[$post->ID] .= ': '. $comments_number;
												$body_fields[$post->ID] .= '</div>';
											}
											if( in_array( 'started', $cols ) ) {
												$body_fields[$post->ID] .= '<div>';
												$body_fields[$post->ID] .= __( 'Started', 'wp-discussion-board' );
												$body_fields[$post->ID] .= ': '. $started_time;
												$body_fields[$post->ID] .= '</div>';
											}
											if( in_array( 'voices', $cols ) ) {
												$body_fields[$post->ID] .= '<div>';
												$body_fields[$post->ID] .= __( 'Voices', 'wp-discussion-board' );
												$body_fields[$post->ID] .= ': '. absint( $voices );
												$body_fields[$post->ID] .= '</div>';
											}
											$mobile_body_fields = '';
											$mobile_body_fields = apply_filters( 'ctdb_mobile_body_fields', $mobile_body_fields, $post->ID, $cols );
											$body_fields[$post->ID] .= $mobile_body_fields;
										$body_fields[$post->ID] .= '</div>';

									} else if( $col == 'replies' ) {
										$body_fields[$post->ID] .= $comments_number;
									} else if( $col == 'started' ) {
										$body_fields[$post->ID] .= $started_time;
									} else if( $col == 'voices' ) {
										$body_fields[$post->ID] .= absint( $voices );
									}

									// Filter here for additional columns
									$body_fields = apply_filters( 'ctdb_topic_column', $body_fields, $post->ID, $col );

									$body_fields[$post->ID] .= '</li>';

								}

							}

							$body_fields[$post->ID] .= '</ul><!-- ctdb-topic-table-row -->';

						}

					endwhile;

					$body_fields = apply_filters( 'ctdb_topic_body_fields', $body_fields, $topics, $titles, $cols );

					if( ! empty( $body_fields ) ) {
						$output .= join( '', $body_fields );
					}

				}

				$output .= '</li><!-- .ctdb-topic -->';
			$output .= '</ul><!-- ctdb-topic-table-body -->';

		}

		return $output;
	}

	/**
	 * Add the New Topic button to a taxonomy-board.php template page
	 * @since 1.5.0
	 */
	public function add_button_taxonomy_board() {
		// Get the board ID
		$board_id = get_queried_object()->term_id;
		if( ! empty( $board_id ) ) {
			echo '<p>' . do_shortcode( '[new_topic_button board=' . intval( $board_id ) . ']' ) . '</p>';
		}
	}

	/**
	 * Filter the New Topic button URL
	 * We can add a board parameter
	 * @since 1.5.0
	 */
	public function filter_new_topic_button_url( $url, $atts, $new_topic_page ) {
		// Check for the board parameter
		if( isset( $atts['board'] ) ) {
			$board_id = $atts['board'];
			// Check that $board_id is actually a board ID
			$id_exists = get_term_by( 'id', $board_id, 'board' );
			if( $id_exists ) {
				$url = add_query_arg(
					array(
						'board_id' => intval( $board_id )
					),
					get_permalink( $new_topic_page )
				);
			}

		}
		return $url;
	}

	/**
	 * Register the taxonomy
	 * @since 1.5.0
	 */
	public function register_taxonomy() {

		// Add Board taxonomy
		// Only if option is selected
		$options = get_option ( 'ctdb_boards_settings' );
		if ( isset ( $options['enable_boards'] ) && ! empty( $options['enable_boards'] ) ) {
			$labels = array(
				'name'						=> _x( 'Boards', 'taxonomy general name', 'discussion-board-pro' ),
				'singular_name'				=> _x( 'Board', 'taxonomy singular name', 'discussion-board-pro' ),
				'search_items'				=> __( 'Search Boards', 'discussion-board-pro' ),
				'all_items'					=> __( 'All Boards', 'discussion-board-pro' ),
				'parent_item'				=> __( 'Parent Board', 'discussion-board-pro' ),
				'parent_item_colon'			=> __( 'Parent Board:', 'discussion-board-pro' ),
				'view_item'					=> __( 'View Board', 'discussion-board-pro' ),
				'edit_item'					=> __( 'Edit Board', 'discussion-board-pro' ),
				'update_item'				=> __( 'Update Board', 'discussion-board-pro' ),
				'add_new_item'				=> __( 'Add New Board', 'discussion-board-pro' ),
				'new_item_name'				=> __( 'New Board Name', 'discussion-board-pro' ),
				'not_found'					=> __( 'No Boards found', 'discussion-board-pro' ),
				'menu_name'					=> __( 'Boards' ),
			);
			$args = array(
				'hierarchical'          => true,
				'labels'                => $labels,
				'show_ui'               => true,
				'show_admin_column'     => true,
				'query_var'             => true,
				'rewrite'               => array( 'slug' => 'board' ),
			);
			register_taxonomy( 'board', 'discussion-topics', $args );
		}

	}

	// Remove the default Board metabox
	// In order to replace it with radio button version
	public function remove_meta_box() {
		remove_meta_box( 'boarddiv', 'discussion-topics', 'normal' );
	}

	// Add radio button version of the metabox
	public function add_meta_box() {
		// If there is no board ID, add a select so that the user can pick one
		$options = get_option( 'ctdb_boards_settings' );

		// Add a select box for categories
		// @todo Display as checkboxes / radio buttons / multiselect ?
		if( ! empty( $options['enable_boards'] ) ) {
			add_meta_box( 'board_taxonomy_div', __( 'Boards', 'discussion-board-pro' ), array( $this, 'board_metabox' ), 'discussion-topics', 'side', 'high' );
		}
	}

	// Output radio button version
	// @credit https://code.tutsplus.com/articles/how-to-use-radio-buttons-with-taxonomies--wp-24779
	public function board_metabox( $post ) {
		//Get taxonomy and terms
    $taxonomy = 'board';

    //Set up the taxonomy object and get terms
    $tax = get_taxonomy($taxonomy);
    $terms = get_terms($taxonomy,array('hide_empty' => 0));

    //Name of the form
    $name = 'tax_input[' . $taxonomy . ']';

		//Get current and popular terms
		$popular = get_terms( $taxonomy, array( 'orderby' => 'count', 'order' => 'DESC', 'number' => 10, 'hierarchical' => false ) );
		$postterms = get_the_terms( $post->ID,$taxonomy );
		$current = '';
		if( ! empty( $postterms ) && is_array( $postterms ) ) {
			$current = ( $postterms ? array_pop( $postterms ) : false);
			$current = ( $current ? $current->term_id : 0);
		} ?>

    <div id="taxonomy-<?php echo $taxonomy; ?>" class="categorydiv">

        <!-- Display taxonomy terms -->
        <div id="<?php echo $taxonomy; ?>-all">
            <ul id="<?php echo $taxonomy; ?>checklist" class="list:<?php echo $taxonomy?> categorychecklist form-no-clear">
                <?php if( ! empty( $terms ) && is_array( $terms ) ) {
					foreach( $terms as $term ){
						$id = $taxonomy.'-'.$term->term_id;
						echo "<li id='$id'><label class='selectit'>";
						echo "<input type='radio' id='in-$id' name='{$name}'" . checked( $current, $term->term_id, false ) . "value='$term->term_id' />$term->name<br />";
						echo "</label></li>";
	                }
				} else {
					// There are no boards set yet
					_e( 'No boards found.', 'discussion-board-pro' );
				} ?>
           </ul>
        </div>

    </div>
    <?php
	}

	// Save the board
	public function save_board( $post_id ) {
		if( ! empty( $_POST['tax_input']['board'] ) ) {
			wp_set_post_terms( $post_id, $_POST['tax_input']['board'], 'board' );
		}
	}

	/**
	 * Update the board's follow/unfollow status
	 * Add or remove user's email address from ctdb_board_followers
	 * @since 1.6.2
	 */
	public function update_board_follow_status() {

		$boardid = absint( $_POST['boardid'] );
		check_ajax_referer( 'board_follow_nonce_' . $boardid, 'security' );

		$return = 'fail';
		if ( ! empty( $_POST['userid'] ) && ! empty( $_POST['boardid'] ) && ! empty( $_POST['following'] ) ) {

			$userid = absint( $_POST['userid'] );
			$following = stripslashes( strip_tags( $_POST['following'] ) );

			// We need to update the post meta field
			$followers = get_term_meta( $boardid, 'ctdb_board_followers', true );
			if( ! empty( $followers ) ) {
				// Convert to an array
				$followers = explode( ',', $followers );
			} else {
				$followers = array();
			}

			// Current user email address
			$current_user = wp_get_current_user();
			$email_address = $current_user->user_email;

			// Either following or unfollowing
			if( $following == 'follow' ) {
				// Add email address
				$followers[] = $email_address;
			} else {
				// Remove email address
				$followers = array_diff( $followers, $email_address );
			}

			// Back to a string
			$followers = join( ',', $followers );
			// Update the board with the new list of followers
			update_term_meta( $boardid, 'ctdb_board_followers', $followers );

			$return = 'done';
		}
		echo $following;
		die();
	}

}

$CT_DB_Pro_Boards = new CT_DB_Pro_Boards();
$CT_DB_Pro_Boards -> init();
